Glydo.VerticalPaneController = Prototype.Class.create(Glydo.EventSource,{
	initialize: function($super,recVariantClassesMap) {
		$super();
		this.isInitialized = false;
		this.randomStylesPerN = ({});
		this.recVariantClassesMap = Prototype.O.extend({},recVariantClassesMap);
		this.contextRecommendationLockers = 0;
	},

	init: function(idPrefix,application,filter,adFilter,name) {
		this.isInitialized = true;
		this.idPrefix = idPrefix;
		this.name = name;
		this.localizedStrings = new Glydo.Utils.LocalizedBundle(document);
		this.application = application;
		this.filter = filter;
		this.adFilter = adFilter;
		this.recommendationsData = [];
		this.documentManager = this.application.documentManager;
		this.documentManager.addListener(this);
		this.recommendationsView = this.element('recommendations-view');
		this.contextRecommendation = null;
		this.lockedContextRecommendation = null;
		this.setupEventHandlers();
		this.generateRendition();
	},

	destroy: function() {
		this.clearRendition();
		this.documentManager.removeListener(this);
		this.isInitialized = false;
	},

	onVisible: function() {
		this.recommendationsData.forEach(function(recData) {
			if (recData.mainNode && recData.mainNode.render) {
				recData.mainNode.render();
			}
		},this)
	},
	
	element: function(id) {
		return document.getElementById(this.getId(id));
	},

	getId: function(id) {
		return "glydo-" + this.idPrefix + "-" + id;
	},

	recIdPrefix: function(rec) {
		return this.getId("recommendation-" + rec.globalId);
	},

	setupEventHandlers: function() {
	},

	canRender: function() {
		return this.isInitialized;
	},

	clearRendition: function() {
		// We remove the variant class name from all recommendations
		// to overcome a mozilla bug, where bindings are not detached properly
		// if an element is removed from the document without destroying it. 
		// FIXME: This still doesn't work, and there's a chance of a leak here.
		var nRecs = this.recommendationsData.length;
		for (var i = 0; i < nRecs; ++i) {
			var recData = this.recommendationsData[i];
			Prototype.E.removeClassName(recData.mainNode,recData.variantClass);
			Prototype.E.removeClassName(recData.mainNode,recData.styleClass);
		}
		while (this.recommendationsView.hasChildNodes()) {
			this.recommendationsView.removeChild(this.recommendationsView.firstChild);
		}
		this.recommendationsData = [];
		delete this.documentRenditionData;
	},

	generateRendition: function() {
		if (!this.canRender()) {
			return;
		}
		this.clearRendition();
		var current = this.documentManager.getCurrentProcessedDocument();
		if (current) {
			this.renderDocument(current);
		} else {
			this.renderNoDocument();
		}
		this.fire("onPaneContentsChanged",this.name,this.recommendationsData.length);
	},

	onCurrentProcessedDocumentChanged: function(newDocumentEntry) {
		this.generateRendition();
	},

	renderDocument: function(documentEntry) {
		if (!documentEntry) {
			return;
		}
		// Get or create the document rendition data
		this.documentRenditionData = {
				documentEntry: documentEntry,
				renderedRecommendations: false,
				recommendationsData: []
		};
		// Create a new element for the document
		this.documentRenditionData.recsNode = document.createElement('vbox');
		this.documentRenditionData.recsNode.className = "glydo-recommendations-document";
		// Insert the new document element
		this.recommendationsView.appendChild(this.documentRenditionData.recsNode);
		this.renderDocumentRecommendations(this.documentRenditionData);
	},

	renderDocumentRecommendations: function(documentRenditionData) {
		var parent = documentRenditionData.recsNode;
		var documentEntry = documentRenditionData.documentEntry;
		if (!documentEntry || !parent) {
			return;
		}
		// Empty the parent in case it has previous content
		parent.textContent = "";
		if (!documentEntry.isDone()) {
			this.renderDocumentProcessing(documentRenditionData);
			return;
		}
		if (documentEntry.isFailed()) {
			this.renderDocumentError(documentRenditionData);
			return;
		}
		var recommendations = documentEntry.selectRecommendations(this.filter);
		Prototype.E.show(parent);
		if (!recommendations || (recommendations.length == 0)) {
			this.renderDocumentNoRecommendations(documentRenditionData);
			return;
		}
		var nRecs = recommendations.length;
		for (var r = 0; r < nRecs; ++r) {
			var rec = recommendations[r];
			var recData = this.renderRecommendation(parent,rec,r,nRecs);
			recData.mainNode.setAttribute("realEstatePosition","regular[" + r + "]");
			this.recommendationsData.push(recData);
			documentRenditionData.recommendationsData.push(recData);
		}
		if (this.adFilter) {
			var otherRecommendations = documentEntry.selectRecommendations(this.adFilter);
			if (otherRecommendations.length > 0) {
				Glydo.Utils.shuffle(otherRecommendations);
				var rec = otherRecommendations[0];
				var recData = this.renderRecommendation(parent,rec,nRecs+1,otherRecommendations.length);
				recData.mainNode.setAttribute("realEstatePosition","bottom_ads[0]");
				this.recommendationsData.push(recData);
				documentRenditionData.recommendationsData.push(recData);
			}
		}
		documentRenditionData.renderedRecommendations = true;
	},

	renderDocumentProcessing: function(documentRenditionData) {
		var processingNode = document.createElement("description");
		processingNode.setAttribute("value",this.localizedStrings.get("vertical_pane.processing.label"));
		processingNode.className = "glydo-recommendations-document-processing";
		documentRenditionData.recsNode.appendChild(processingNode);
	},

	renderDocumentError: function(documentRenditionData) {
		this.renderDocumentNoRecommendations(documentRenditionData);
	},
	
	renderNoDocument: function() {
		var noDocNode = document.createElement("description");
		noDocNode.setAttribute("value",this.localizedStrings.get("vertical_pane.no_document.label"));
		noDocNode.className = "glydo-recommendations-no-document";
		this.recommendationsView.appendChild(noDocNode);
	},

	renderDocumentNoRecommendations: function(documentRenditionData) {
		var noRecsNode = document.createElement("description");
		noRecsNode.setAttribute("value",this.localizedStrings.get("vertical_pane.no_recommendations.label"));
		noRecsNode.className = "glydo-recommendations-document-no-recommendations";
		documentRenditionData.recsNode.appendChild(noRecsNode);
	},

	getVariantClass: function(rec) {
		return this.recVariantClassesMap.lookup(rec);
	},
	
	renderRecommendation: function(parent,rec,index,total) {
		var nStyles = this.recVariantClassesMap.nStyles(rec);
		var randomStyles = this.randomStylesPerN[nStyles];
		if (!randomStyles) {
			randomStyles = [];
			for (var i = 0; i < nStyles; ++i) {
				randomStyles[i] = i;
			}
			Glydo.Utils.shuffle(randomStyles);
			this.randomStylesPerN[nStyles] = randomStyles;
		}
		var styleClass = "glydo-style-" + randomStyles[(index % nStyles)];
		var recData = ({
				rec: rec,
				variantClass: this.getVariantClass(rec),
				styleClass: styleClass
			});
			recData.mainNode = document.createElement('glydorecommendation');
			Prototype.E.addClassName(recData.mainNode,recData.variantClass);
			Prototype.E.addClassName(recData.mainNode,recData.styleClass);
//			recData.mainNode.contextMenu = "glydo-recommendation-context-menu";
			recData.mainNode.setAttribute("sharingpopup","glydo-sharing-recommendation-popup");

			recData.mainNode.addEventListener("GlydoBindingLoaded",Prototype.F.bindAsEventListener(Prototype.F.curry(this.onRecommendationBindingLoaded,recData),this),false);

			// We need to append the child only after we set the event handler to make sure we get the binding loaded even
			// in any case
			parent.appendChild(recData.mainNode);
			return recData;
	},

	onRecommendationBindingLoaded: function(recData,event) {
		if (event.originalTarget === recData.mainNode) {
			recData.mainNode.rec = recData.rec;
		}
	},		

	onContextMenu: function(event) {
		if (event.type == "popupshowing") {
			this.lockContextRecommendation();
		} else if (event.type == "popuphidden") {
			this.unlockContextRecommendation();
		}
		return true;
	},

	lockContextRecommendation: function() {
		if (this.contextRecommendationLockers <= 0) {
			// Freeze hover effects for all recs
			this.recommendationsData.forEach(function(recData){
				recData.mainNode.freezeHover = true;
				if (recData.mainNode.hover) {
					this.lockedContextRecommendation = recData;
				}
			},this)
			// Sanity
			this.contextRecommendationLockers = 0;
		}
		++this.contextRecommendationLockers;
	},
	
	unlockContextRecommendation: function() {
		--this.contextRecommendationLockers;
		if (this.contextRecommendationLockers <= 0) {
			// Unfreeze hover effects for all recs
			this.recommendationsData.forEach(function(recData){
				recData.mainNode.freezeHover = false;
			},this)
			this.lockedContextRecommendation = null;
			// Sanity
			this.contextRecommendationLockers = 0;
		}
	},
	
	getContextRecommendation: function() {
		var recData = this.lockedContextRecommendation || this.contextRecommendation;
		return recData === null ? null : ({
			rec: recData.rec,
			node: recData.mainNode
		});
	}
	
});

